@@ -1,27 +1,19 @@
 from __future__ import annotations
 
-from dataclasses import MISSING, Field, asdict, dataclass, field, fields, is_dataclass
+from dataclasses import MISSING, asdict, fields, is_dataclass
 from types import ModuleType
-from typing import Any, Dict, Generic, List, Tuple, Type, Union
+from typing import Any, Dict, Generic, List, Mapping, Optional, Tuple, Union
 
+import pytest
 from pydantic import create_model
 from pydantic.error_wrappers import ValidationError
 from pydantic.types import StrictBool, StrictBytes, StrictFloat, StrictInt, StrictStr
 
+from pytest_embrace.case import CaseTypeInfo
+
 from .anno import get_pep_593_values
 from .case import CaseType, Trickle
-from .exc import CaseConfigurationError
-
-
-@dataclass
-class AttrInfo:
-    dc_field: Field
-    module_value: Any
-    name: str = field(init=False)
-
-    def __post_init__(self) -> None:
-        self.name = self.dc_field.name
-
+from .exc import CaseConfigurationError, EmbraceError
 
 ShouldBecomeStrictBuiltinTypes = Union[str, bytes, int, float, bool]
 StrictPydanticTypes = Union[StrictStr, StrictBytes, StrictInt, StrictFloat, StrictBool]
@@ -41,25 +33,12 @@ PYDANTIC_STRICTIFICATION_MAP = {
 }
 
 
-# TODO should take a moduleinfo instead of instantiating
-def load(cls: Type[CaseType], module: ModuleType) -> List[CaseType]:
-    module_info = ModuleInfo(case_type=cls, module=module)
-    if module_info.table is not None:
-        module_info.trickle_down_table()
-        return [
-            revalidate_dataclass(case, alias=f"{module_info}.table[{i}]{case}")
-            for i, case in enumerate(module_info.table)
-        ]
-
-    return [revalidate_dataclass(module_info.to_case(), alias=str(module_info))]
-
-
 class ModuleInfo(Generic[CaseType]):
-    def __init__(self, *, case_type: Type[CaseType], module: ModuleType):
-        self.case_type = case_type
+    def __init__(self, *, case_type_info: CaseTypeInfo, module: ModuleType) -> None:
+        self.case = case_type_info
         self.module = module
         self.name = module.__name__
-        self.cls_fields = {field.name: field for field in fields(self.case_type)}
+        self.cls_fields = {field.name: field for field in fields(self.case.type)}
         self.filename_values = {
             k: derive.get_attr_value(module.__name__)
             for k, v in self.cls_fields.items()
@@ -72,63 +51,62 @@ class ModuleInfo(Generic[CaseType]):
         }
         self.module_attrs.update(self.filename_values)
 
-        if "table" in dir(module):
-            self.table = getattr(module, "table")
-            is_valid_table = (
-                isinstance(self.table, list)
-                and len(self.table) > 0
-                # would love to use an isinstance() here, but somehow
-                # (in Pytester tests specifically)
-                # it managed to not work? despite extensive time in PDB?
-                and all(str(type(x)) == str(self.case_type) for x in self.table)
-            )
-            if not is_valid_table:
-                raise
-        else:
-            self.table = None
-
-        self.pep_539 = get_pep_593_values(self.case_type)
-        UNSET = object()
-        self.trickle_defaults: Dict[str, Tuple[Any, Trickle]] = {
-            k: (self.module_attrs.get(k, UNSET), trickle)
-            for k, v in self.cls_fields.items()
-            if (trickle := v.metadata.get("trickle")) is not None
-        }
-        self.all_trickles_unset = len(self.trickle_defaults) and all(
-            v is UNSET for v, _ in self.trickle_defaults.values()
+        self.table = getattr(module, "table", None)
+        self.has_table_of_tests = (
+            isinstance(self.table, list)
+            and len(self.table) > 0
+            # would love to use an isinstance() here, but somehow
+            # (in Pytester tests specifically)
+            # it managed to not work? despite extensive time in PDB?
+            and all(str(type(x)) == str(self.case.type) for x in self.table)
         )
 
+        self.pep_539 = get_pep_593_values(self.case.type)
+
+        if self.has_table_of_tests:
+            self._trickle_down_table()
+
     def __str__(self) -> str:
         return f"Module[{self.name}]"
 
     def to_case(self) -> CaseType:
         kwargs = {k: v for k, v in self.module_attrs.items() if k != "table"}
         try:
-            return self.case_type(**kwargs)
+            return self.case.type(**kwargs)
         except TypeError:
             raise CaseConfigurationError(
                 f"Incorrect or missing attributes in {self.name}."
                 f"\nOnly got attributes {kwargs}"
             )
 
-    def trickle_down_table(self) -> None:
+    def _trickle_down_table(self) -> None:
         assert self.table is not None
+        UNSET = object()
+        trickle_defaults: Dict[str, Tuple[Any, Trickle]] = {
+            k: (self.module_attrs.get(k, UNSET), trickle)
+            for k, v in self.cls_fields.items()
+            if (trickle := v.metadata.get("trickle")) is not None
+        }
+        all_trickles_unset = len(trickle_defaults) and all(
+            v is UNSET for v, _ in trickle_defaults.values()
+        )
+
         for i, case in enumerate(self.table):
             for k, v in asdict(case).items():
                 if k in self.filename_values:
                     setattr(case, k, self.filename_values[k])
-                elif self.all_trickles_unset and isinstance(v, Trickle):
+                elif all_trickles_unset and isinstance(v, Trickle):
                     raise CaseConfigurationError(
                         f"'{k}' is unset at the module level and in table[{i}]:{case}."
                         " '{k}' is marked as 'trickles()' so it must be set somewhere."
                     )
-                elif k in self.trickle_defaults and isinstance(v, Trickle):
+                elif k in trickle_defaults and isinstance(v, Trickle):
                     # absorb the default trickle value
-                    (trickle_value, _) = self.trickle_defaults[k]
+                    (trickle_value, _) = trickle_defaults[k]
                     setattr(case, k, trickle_value)
-                elif k in self.trickle_defaults:
+                elif k in trickle_defaults:
                     # there was a trickle, but it was overridden
-                    (_, trickle_config) = self.trickle_defaults[k]
+                    (_, trickle_config) = trickle_defaults[k]
                     if trickle_config.no_override and k in self.module_attrs:
                         raise CaseConfigurationError(
                             f"Trickle-down attribute '{k}"
@@ -136,6 +114,16 @@ class ModuleInfo(Generic[CaseType]):
                         )
 
 
+def load(test: ModuleInfo[CaseType]) -> List[CaseType]:
+    if test.table is not None:
+        return [
+            revalidate_dataclass(case, alias=f"{test}.table[{i}]{case}")
+            for i, case in enumerate(test.table)
+        ]
+
+    return [revalidate_dataclass(test.to_case(), alias=str(test))]
+
+
 def _raise_non_dataclass(o: object) -> None:
     if not is_dataclass(o):
         raise CaseConfigurationError("Must use a dataclass for case object.")
@@ -189,3 +177,19 @@ def revalidate_dataclass(case: CaseType, *, alias: str) -> CaseType:
         _report_validation_error(e, target_name=alias)
 
     return case
+
+
+def find_embrace_requester(
+    *, metafunc: pytest.Metafunc, registry: Mapping[str, CaseTypeInfo]
+) -> Optional[ModuleInfo]:
+    potentials = [name for name in metafunc.fixturenames if name in registry]
+
+    if len(potentials) > 1:
+        raise EmbraceError(f"Can't request multiple Embrace fixtures. Got {potentials}")
+
+    found = next(iter(potentials), None)
+    return (
+        found
+        if found is None
+        else ModuleInfo(case_type_info=registry[found], module=metafunc.module)
+    )
